@@ -13,20 +13,22 @@
 # limitations under the License.
 
 import logging
+from datetime import datetime, timedelta
 
 import braintree
 from braintree.exceptions import (AuthenticationError, AuthorizationError,
                                   DownForMaintenanceError, ServerError,
                                   UpgradeRequiredError)
+import dateutil.parser
 from django_fsm import TransitionNotAllowed
 
-
+from silver.models import Transaction
 from silver.payment_processors import PaymentProcessorBase, get_instance
 from silver.payment_processors.forms import GenericTransactionForm
 from silver.payment_processors.mixins import TriggeredProcessorMixin
 
 from silver_braintree.models import BraintreePaymentMethod
-from silver_braintree.models.customer_data import CustomerData
+from silver_braintree.models import CustomerData
 from silver_braintree.views import BraintreeTransactionView
 
 
@@ -96,6 +98,7 @@ class BraintreeTriggeredBase(PaymentProcessorBase, TriggeredProcessorMixin):
 
         if instrument_type == payment_method.Types.PayPal:
             payment_method_details['email'] = result_details.payer_email
+            payment_method.display_info = result_details.payer_email
         elif instrument_type == payment_method.Types.CreditCard:
             payment_method_details.update({
                 'card_type': result_details.card_type,
@@ -114,7 +117,7 @@ class BraintreeTriggeredBase(PaymentProcessorBase, TriggeredProcessorMixin):
 
     def _update_transaction_status(self, transaction, result_transaction):
         """
-        :param transaction: A Transaction.
+        :param transaction: A Silver transaction with a Braintree payment method.
         :param result_transaction: A transaction from a braintreeSDK
                                    result(response).
         :description: Updates a given transaction's data with data from a
@@ -127,7 +130,10 @@ class BraintreeTriggeredBase(PaymentProcessorBase, TriggeredProcessorMixin):
         transaction.external_reference = result_transaction.id
         status = result_transaction.status
 
-        transaction.data['status'] = status
+        transaction.data.update({
+            'status': status,
+            'braintree_id': result_transaction.id
+        })
 
         target_state = None
         try:
@@ -139,7 +145,7 @@ class BraintreeTriggeredBase(PaymentProcessorBase, TriggeredProcessorMixin):
                 target_state = transaction.States.Failed
                 if transaction.state != target_state:
                     fail_code = self._get_silver_fail_code(result_transaction)
-                    fail_reason = self._get_braintree_fail_code(result_transaction)
+                    fail_reason = self._get_braintree_transaction_fail_code(result_transaction)
                     transaction.fail(fail_code=fail_code, fail_reason=fail_reason)
                     return False
 
@@ -179,9 +185,9 @@ class BraintreeTriggeredBase(PaymentProcessorBase, TriggeredProcessorMixin):
     def _get_errors(self, result):
         return [
             error.code for error in result.errors.deep_errors
-        ] if result.errors else None
+        ] if result.errors else self._get_braintree_transaction_fail_code(result.transaction)
 
-    def _get_braintree_fail_code(self, result_transaction):
+    def _get_braintree_transaction_fail_code(self, result_transaction):
         if result_transaction.status in (
                 braintree.Transaction.Status.ProcessorDeclined,
                 braintree.Transaction.Status.AuthorizationExpired
@@ -195,11 +201,16 @@ class BraintreeTriggeredBase(PaymentProcessorBase, TriggeredProcessorMixin):
             return result_transaction.processor_settlement_response_code,
 
     def _get_silver_fail_code(self, result_transaction):
-        braintree_fail_code = self._get_braintree_fail_code(result_transaction)
+        braintree_fail_code = self._get_braintree_transaction_fail_code(result_transaction)
 
         if not braintree_fail_code:
             return 'default'
 
+        try:
+            braintree_fail_code = int(braintree_fail_code)
+        except (TypeError, ValueError):
+            return 'default'
+
         if braintree_fail_code in [2001]:
             return 'insufficient_funds'
         elif braintree_fail_code in [2022]:
@@ -224,7 +235,7 @@ class BraintreeTriggeredBase(PaymentProcessorBase, TriggeredProcessorMixin):
 
     def _charge_transaction(self, transaction):
         """
-        :param transaction: The transaction to be charged. Must have a useable
+        :param transaction: The Silver transaction to be charged. Must have a usable Braintree
                             payment_method.
         :return: True on success, False on failure.
         """
@@ -287,27 +298,36 @@ class BraintreeTriggeredBase(PaymentProcessorBase, TriggeredProcessorMixin):
             })
 
         # send transaction request
-        result = braintree.Transaction.sale(payload)
+        transaction.data['requested_at'] = datetime.utcnow().isoformat()
+        transaction.save()
 
-        # handle response
-        if not result.is_success or not result.transaction:
-            errors = self._get_errors(result)
-            logger.warning('Couldn\'t charge Braintree transaction.: %s', {
-                'message': result.message,
-                'errors': errors,
-                'customer_id': customer.id,
-                'card_verification': (result.credit_card_verification if errors
-                                      else None)
-            })
+        try:
+            result = braintree.Transaction.sale(payload)
 
-            transaction.data['error_codes'] = errors
-            try:
-                fail_code = self._get_silver_fail_code(result.transaction)
+            # handle response
+            if not result.is_success or not result.transaction:
+                errors = self._get_errors(result)
+                logger.warning('Couldn\'t charge Braintree transaction.: %s', {
+                    'message': result.message,
+                    'errors': errors,
+                    'customer_id': customer.id,
+                    'card_verification': (result.credit_card_verification if errors
+                                          else None)
+                })
+
+                transaction.data['error_codes'] = errors
+
+                if result.transaction:
+                    transaction.data['response_code'] = self._get_braintree_transaction_fail_code(
+                        result.transaction
+                    )
+                fail_code = (self._get_silver_fail_code(result.transaction) if result.transaction
+                             else 'default')
                 transaction.fail(fail_code=fail_code, fail_reason=errors)
-            finally:
-                transaction.save()
 
                 return False
+        finally:
+            transaction.save()
 
         self._update_customer(customer, result.transaction.customer_details)
 
@@ -339,29 +359,68 @@ class BraintreeTriggeredBase(PaymentProcessorBase, TriggeredProcessorMixin):
 
     def execute_transaction(self, transaction):
         """
-        :param transaction: A Braintree transaction in Initial state.
-        :return: True on success, False on failure.
+        :param transaction: A Silver transaction with a Braintree payment method, in Initial state.
+        :return: True if the transaction was successfully sent to processing, False otherwise.
         """
 
         payment_processor = get_instance(transaction.payment_processor)
         if not payment_processor == self:
             return False
 
-        if transaction.state != transaction.States.Initial:
-            return False
-
-        try:
-            transaction.process()
-            transaction.save()
-        except TransitionNotAllowed:
+        if transaction.state != transaction.States.Pending:
             return False
 
         return self._charge_transaction(transaction)
 
+    def recover_lost_transaction_id(self, transaction):
+        """
+        :param transaction: A Silver transaction with a Braintree payment method.
+        :return: True if the transaction ID was recovered, False otherwise.
+        """
+        if transaction.data.get('braintree_id'):
+            return True
+
+        search_result = braintree.Transaction.search(
+            braintree.TransactionSearch.amount.is_equal(transaction.amount),
+            braintree.TransactionSearch.payment_method_token.is_equal(
+                transaction.payment_method.token
+            ),
+            braintree.TransactionSearch.created_at.between(
+                dateutil.parser.parse(transaction.data['requested_at']) - timedelta(seconds=1),
+                transaction.created_at + timedelta(seconds=61)
+            )
+        )
+
+        transaction_list = list(search_result.items)
+
+        # get rid of transactions that are already tracked before trying to find a match
+        if len(transaction_list) > 1:
+            transaction_list = [
+                transaction for transaction in transaction_list
+                if not Transaction.objects.filter(external_reference=transaction.id).exists()
+            ]
+
+        # there was a single match
+        if len(transaction_list) == 1:
+            transaction.data['braintree_id'] = transaction_list[0].id
+            transaction.external_reference = transaction_list[0].id
+
+            return True
+
+        # there were no transactions that could match our transaction
+        if not transaction_list:
+            transaction.fail(
+                fail_reason="The transaction request didn't reach Braintree."
+            )
+            transaction.save()
+
+        # if there are 2 or more potential matches, no action is taken
+        return False
+
     def fetch_transaction_status(self, transaction):
         """
-        :param transaction: A Braintree transaction in Initial or Pending state.
-        :return: True on success, False on failure.
+        :param transaction: A Silver transaction with a Braintree payment method, in Pending state.
+        :return: True if the transaction status was updated, False otherwise.
         """
 
         payment_processor = get_instance(transaction.payment_processor)
@@ -372,13 +431,14 @@ class BraintreeTriggeredBase(PaymentProcessorBase, TriggeredProcessorMixin):
             return False
 
         if not transaction.data.get('braintree_id'):
-            logger.warning('Found pending Braintree transaction with no '
-                           'braintree_id: %s', {
-                                'transaction_id': transaction.id,
-                                'transaction_uuid': transaction.uuid
-                           })
+            if not self.recover_lost_transaction_id(transaction):
+                logger.warning('Found pending Braintree transaction with no '
+                               'braintree_id: %s', {
+                                    'transaction_id': transaction.id,
+                                    'transaction_uuid': transaction.uuid
+                               })
 
-            return False
+                return False
 
         try:
             result_transaction = braintree.Transaction.find(
@@ -432,7 +492,7 @@ class BraintreeTriggeredBase(PaymentProcessorBase, TriggeredProcessorMixin):
         # manage the transaction
         payment_processor = get_instance(payment_method.payment_processor)
 
-        if not payment_processor.execute_transaction(transaction):
+        if not payment_processor.process_transaction(transaction):
             try:
                 transaction.fail()
                 transaction.save()
