@@ -11,19 +11,35 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-import factory
 
-from silver.tests.factories import TransactionFactory, CustomerFactory
+import factory
+from factory.django import mute_signals, DjangoModelFactory
+
+from django.db.models import signals
+
+from silver.fixtures.factories import CustomerFactory, TransactionFactory
 
 from silver_braintree.models import BraintreePaymentMethod
 
 
-class BraintreePaymentMethodFactory(factory.DjangoModelFactory):
+@mute_signals(signals.pre_save, signals.post_save)
+class BraintreePaymentMethodFactory(DjangoModelFactory):
     class Meta:
         model = BraintreePaymentMethod
 
     payment_processor = 'BraintreeTriggered'
     customer = factory.SubFactory(CustomerFactory)
+    data = factory.Sequence(lambda i: {})
+
+
+@mute_signals(signals.pre_save, signals.post_save)
+class BraintreeRecurringPaymentMethodFactory(DjangoModelFactory):
+    class Meta:
+        model = BraintreePaymentMethod
+
+    payment_processor = 'BraintreeTriggeredRecurring'
+    customer = factory.SubFactory(CustomerFactory)
+    data = factory.Sequence(lambda i: {})
 
 
 class BraintreeTransactionFactory(TransactionFactory):
